package com.yaozhitech.spring5.intercept;

import java.util.Arrays;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import com.yaozhitech.spring5.annotation.IgnoreClientToken;
import com.yaozhitech.spring5.common.exception.auth.ClientForbiddenException;
import com.yaozhitech.spring5.config.ServiceAuthConfig;
import com.yaozhitech.spring5.utils.JwtUtils;

import lombok.extern.slf4j.Slf4j;

/**
 * 微服务之间的认证
 * @author EDZ
 *
 */
@Slf4j
public class ServiceAuthRestInterceptor extends HandlerInterceptorAdapter {
    private Logger logger = LoggerFactory.getLogger(ServiceAuthRestInterceptor.class);

//  @Autowired
//  private ServiceAuthUtil serviceAuthUtil;
//
//  @Autowired
  private ServiceAuthConfig serviceAuthConfig;

  private List<String> allowedClient = Arrays.asList("admin", "order", "gateway");

  @Override
  public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
		HandlerMethod handlerMethod = (HandlerMethod) handler;
		// ip
//		if (ClientUtil.isLocalhost(request)) {
//			return super.preHandle(request, response, handler);
//		}
		
      // 配置该注解，说明不进行服务拦截
      IgnoreClientToken annotation = handlerMethod.getBeanType().getAnnotation(IgnoreClientToken.class);
      if (annotation == null) {
          annotation = handlerMethod.getMethodAnnotation(IgnoreClientToken.class);
      }
      if(annotation != null) {
          return super.preHandle(request, response, handler);
      }

		String token = request.getHeader("x-auth-client");
		logger.info(token);
		
		if (token.equals("gateway")) {
			return super.preHandle(request, response, handler);
		}

		try {
			String uniqueName = JwtUtils.getUsername(token);
			for (String client : allowedClient) {
				if (client.equals(uniqueName.split("\\.")[0])) {
					return super.preHandle(request, response, handler);
				}
			}
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			throw new ClientForbiddenException("Client verfy error");
		}
      
      throw new ClientForbiddenException("Client is Forbidden!");
  }
}
